% % LLS (2017) % %
function [MCsize,maxPloss,refsize]=MCsizepower(alpha,T,DGP,draws,regs,m)
% alpha: parameter of DGP, e.g. AR coefficient
% T: length of sample
% DGP: @name_of_dgp, calls DGP
% draws: number of MC draws
% regs: dimension of DGP (for stochastic regressors, does not include
% mandatory constant). Usually the same as m
% m: number of restrictions to test

% Compute truncation parameters etc.
dof=[8,16,32,64];
BQS=ceil(dof/(5/3));
SNW=ceil(3*T./(2*dof));
Bfourier=dof/2;

% Construct kernels and basis functions
kkvb=kvbkernel(T); %Time domain
Wqs=[QSfreq(T,BQS(1));QSfreq(T,BQS(2));QSfreq(T,BQS(3));QSfreq(T,BQS(4))]; %Frequency domain
knw=[nwkernel(T,SNW(1));nwkernel(T,SNW(2));nwkernel(T,SNW(3));nwkernel(T,SNW(4))]; %Time domain
fbasis=wpbasis(T,max(Bfourier));
cbasis=cosbasis(T,max(dof)+1);
lbasis=lgbasis(T,max(dof)+1);
ssbasis=[imbasis(T,dof(1));imbasis(T,dof(2));imbasis(T,dof(3));imbasis(T,dof(4))];

% Construct Storage
betadev=NaN(draws,regs);
omegastoreQS=NaN(draws,4,regs,regs);
omegastoreNW=NaN(draws,4,regs,regs);
omegastoreEWP=NaN(draws,4,regs,regs);
omegastoreCOS=NaN(draws,4,regs,regs);
omegastoreLG=NaN(draws,4,regs,regs);
omegastoreSS=NaN(draws,4,regs,regs);
omegastoreKVB=NaN(draws,1,regs,regs);
w2astore=NaN(draws,1,1);
w2bstore=NaN(draws,1,1);
plugin=NaN(draws,1);
wa=NaN(regs,1);
wb=NaN(regs,1);

% Seed
rng(715);

for i=1:draws
    
    % Generate data
    [zeta,b,~,data,~]=DGP(T,alpha,m);
    % Store betadev
    betadev(i,:)=b;
    
    % Compute ACF
    ACF=NaN(T-1,regs^2);
    for j=1:T
        ACF(j,:)=reshape(((zeta(j:end,:))'*(zeta(1:end-j+1,:))),regs^2,1)/T; %Compute ACF at lag j without substracting mean - as per Jim's notes
    end
    if regs==2
        ACF=[flipud(ACF(2:end,:));[ACF(:,1),fliplr(ACF(:,2:3)),ACF(:,4)]]; % Duplicate for -j
    elseif regs==1
        ACF=[flipud(ACF(2:end,:));ACF];
    end
    XX=data'*data/T; %Compute Sig_XX
    
    % Calculate LRVs
    omegastoreKVB(i,1,:,:)=XX^-1*reshape([fliplr(kkvb),1,kkvb]*ACF,regs,regs)*XX^-1;
    for d=1:length(dof)
        if m==2 % Accounting for the non-unitary Izz ordinates
            omega=zeros(m); % will collect the summation of weights*Izz
            FT=fft(zeta); % compute FT of zeta, dz
            for l=1:BQS(d) % loop through the ordinates of the periodogram, up to J=BQS(d)
                Izz=FT(l+1,:)'*ctranspose(FT(l+1,:)')/T; % Compute the l+1th ordinate, Izz=dz(w)*conj(dz(w))'/T
                omega=omega+Wqs(d,l)*real(Izz); % add the lth weight times this ordinate to the sum
            end
        else
            FT=fft(zeta);
            omega=Wqs(d,1:BQS(d))*(FT(2:BQS(d)+1,:).*conj(FT(2:BQS(d)+1,:))/T);
        end
        omegastoreQS(i,d,:,:)=(XX^-1*omega*XX^-1); % Compute the variance using X'X and omega
        omegastoreNW(i,d,:,:)=XX^-1*reshape([fliplr(knw(d,1:(length(ACF)-1)/2)),1,knw(d,1:(length(ACF)-1)/2)]*ACF,regs,regs)*XX^-1;
        h=fbasis(1:dof(d),1:size(zeta,1))*zeta/T^.5;
        omegastoreEWP(i,d,:,:)=(XX^-1*1/dof(d)*(h'*h)*XX^-1);
        h=cbasis(2:dof(d)+1,1:size(zeta,1))*zeta/T^.5;
        omegastoreCOS(i,d,:,:)=(XX^-1*1/dof(d)*(h'*h)*XX^-1);
        h=lbasis(2:dof(d)+1,1:size(zeta,1))*zeta/T^.5;
        omegastoreLG(i,d,:,:)=(XX^-1*1/dof(d)*(h'*h)*XX^-1);
        h=ssbasis(sum(dof(1:d-1))+1:sum(dof(1:d)),1:size(zeta,1))*zeta/T^.5;
        omegastoreSS(i,d,:,:)=(XX^-1*1/dof(d)*(h'*h)*XX^-1);
    end
    
    % Calculate w(2)_hat for use in data-dependent critical values - see
    % text for formulae
    if m==2
        corrests=[corr(zeta(2:end,1),zeta(1:end-1,1)),corr(zeta(2:end,2),zeta(1:end-1,2))];
    else
        corrests=corr(zeta(2:end,1),zeta(1:end-1,1));
    end
    wplug=2*diag(corrests)./(1-diag(corrests)).^2;
    plugin(i)=trace(wplug);
    
    MT=ceil(10*(T/200)^(1/3));
    sp=(length(ACF)-1)/2+1;
    j=1:MT;
    for k=1:regs
        wa(k)=j.^2*ACF(sp+1:sp+MT,k^2)/sum(ACF(sp+1:sp+MT,k^2));
    end
    w2astore(i,:,:)=sum(wa);
    
    w=2*pi*(1:MT)/T;
    for k=1:regs
        Iw=sum(repmat(zeta(:,k),1,MT).*cos(repmat(w,T,1).*repmat((1:T)',1,MT))).^2/T+sum(repmat(zeta(:,k),1,MT).*sin(repmat(w,T,1).*repmat((1:T)',1,MT))).^2/T;
        [g]=regress(Iw',[ones(MT,1),w'.^2]);
        wb(k)=-g(2)/g(1);
    end
    w2bstore(i,:,:)=sum(wb);
end
%% Size tests
if m==1
    % Construct t stats
    tQS=T^.5*repmat(betadev,1,length(dof))./squeeze(omegastoreQS(:,:,1,1).^.5);
    tNW=T^.5*repmat(betadev,1,length(dof))./squeeze(omegastoreNW(:,:,1,1).^.5);
    tEWP=T^.5*repmat(betadev,1,length(dof))./squeeze(omegastoreEWP(:,:,1,1).^.5);
    tCOS=T^.5*repmat(betadev,1,length(dof))./squeeze(omegastoreCOS(:,:,1,1).^.5);
    tLG=T^.5*repmat(betadev,1,length(dof))./squeeze(omegastoreLG(:,:,1,1).^.5);
    tSS=T^.5*repmat(betadev,1,length(dof))./squeeze(omegastoreSS(:,:,1,1).^.5);
    tKVB=T^.5*betadev./squeeze(omegastoreKVB(:,:,1,1).^.5);
    
    
    % Critical values
    QScrits=[-2.3386,2.3375;-2.1203,2.1210;-2.0345,2.0338;-1.9963,1.9977,]'; %% Externally calculated, see readme
    QSmat1=repmat(QScrits(1,:),draws,1);
    QSmat2=repmat(QScrits(2,:),draws,1);
    NWcrits=[-2.4918,2.4924;-2.2200,2.2135;-2.0896,2.0797;-2.0219,2.0233]'; %% Externally calculated, see readme
    NWmat1=repmat(NWcrits(1,:),draws,1);
    NWmat2=repmat(NWcrits(2,:),draws,1);
    KVBcrits=[-4.771,4.771]'; % see KVB note
    KVBmat1=repmat(KVBcrits(1,:),draws,1);
    KVBmat2=repmat(KVBcrits(2,:),draws,1);
    basisfncrits=[tinv(.025,dof'),tinv(.975,dof')]'; % exact for fixed-b
    basismat1=repmat(basisfncrits(1,:),draws,1);
    basismat2=repmat(basisfncrits(2,:),draws,1);
    
    % Higher order critical values - explained in text
    crita=repmat(finv(.95,m,dof-m+1),draws,1).*(1+pi^2/6*repmat((dof/T).^2,draws,1).*repmat(w2astore,1,length(dof)));
    critb=repmat(finv(.95,m,dof-m+1),draws,1).*(1+pi^2/6*repmat((dof/T).^2,draws,1).*repmat(w2bstore,1,length(dof)));
    critplug=repmat(finv(.95,m,dof-m+1),draws,1).*(1+pi^2/6*repmat((dof/T).^2,draws,1).*repmat(plugin,1,length(dof)));
    EWPcrita1=-sqrt(crita);
    EWPcrita2=sqrt(crita);
    EWPcritb1=-sqrt(critb);
    EWPcritb2=sqrt(critb);
    EWPcritplug1=-sqrt(critplug);
    EWPcritplug2=sqrt(critplug);
    
    % Rejection Rates
    QSrej=mean((1-(QSmat1<tQS & tQS<QSmat2)));
    NWrej=mean((1-(NWmat1<tNW & tNW<NWmat2)));
    EWPrej=mean((1-(basismat1<tEWP & tEWP<basismat2)));
    COSrej=mean((1-(basismat1<tCOS & tCOS<basismat2)));
    LGrej=mean((1-(basismat1<tLG & tLG<basismat2)));
    SSrej=mean((1-(basismat1<tSS & tSS<basismat2)));
    KVBrej=mean((1-(KVBmat1<tKVB & tKVB<KVBmat2)));
    EWParej=mean((1-(EWPcrita1<tEWP & tEWP<EWPcrita2)));
    EWPbrej=mean((1-(EWPcritb1<tEWP & tEWP<EWPcritb2)));
    EWPplugrej=mean((1-(EWPcritplug1<tEWP & tEWP<EWPcritplug2)));
    MCsize=[QSrej,NWrej,EWPrej,COSrej,LGrej,SSrej,KVBrej];%,EWParej,EWPbrej,EWPplugrej];
    refsize=[EWParej,EWPbrej,EWPplugrej];
    
    % Quantiles (and store in reference matrices for use in SAP
    % calculations)
    QSquantiles=quantile(tQS,[.025,.975]);
    QSmat1=repmat(QSquantiles(1,:),draws,1);
    QSmat2=repmat(QSquantiles(2,:),draws,1);
    NWquantiles=quantile(tNW,[.025,.975]);
    NWmat1=repmat(NWquantiles(1,:),draws,1);
    NWmat2=repmat(NWquantiles(2,:),draws,1);
    EWPquantiles=quantile(tEWP,[.025,.975]);
    EWPmat1=repmat(EWPquantiles(1,:),draws,1);
    EWPmat2=repmat(EWPquantiles(2,:),draws,1);
    COSquantiles=quantile(tCOS,[.025,.975]);
    COSmat1=repmat(COSquantiles(1,:),draws,1);
    COSmat2=repmat(COSquantiles(2,:),draws,1);
    LGquantiles=quantile(tLG,[.025,.975]);
    LGmat1=repmat(LGquantiles(1,:),draws,1);
    LGmat2=repmat(LGquantiles(2,:),draws,1);
    SSquantiles=quantile(tSS,[.025,.975]);
    SSmat1=repmat(SSquantiles(1,:),draws,1);
    SSmat2=repmat(SSquantiles(2,:),draws,1);
    KVBquantiles=quantile(tKVB,[.025,.975]);
    KVBmat1=repmat(KVBquantiles(1),draws,1);
    KVBmat2=repmat(KVBquantiles(2),draws,1);
    %% Construct power curves
    
    % Generate alternatives
    delrange=2:.25:4;
    if isequal(DGP,@ARmean)
        betas=delrange/(1-alpha)/sqrt(T);
    elseif isequal(DGP,@stochasAR)
        rho=alpha^2;
        betas=delrange*sqrt((1+rho)/(1-rho))/sqrt(T);
    elseif isequal(DGP,@ARMA21)
        betas=delrange*sqrt(1.7668)/sqrt(T); % Spectral density from Figure 6 etc.
    end
    
    % Storage
    QSpow=NaN(length(delrange),4);
    NWpow=NaN(length(delrange),4);
    EWPpow=NaN(length(delrange),4);
    COSpow=NaN(length(delrange),4);
    LGpow=NaN(length(delrange),4);
    SSpow=NaN(length(delrange),4);
    KVBpow=NaN(length(delrange),1);
    
    for j=1:length(delrange);
        betatest=betadev+repmat(betas(j),draws,1); % Augment estimated beta deviation from run with beta_alt to calculate power under alterntive
        % Test stats
        tQS=T^.5*repmat(betatest,1,length(dof))./squeeze(omegastoreQS(:,:,1,1).^.5);
        tNW=T^.5*repmat(betatest,1,length(dof))./squeeze(omegastoreNW(:,:,1,1).^.5);
        tEWP=T^.5*repmat(betatest,1,length(dof))./squeeze(omegastoreEWP(:,:,1,1).^.5);
        tCOS=T^.5*repmat(betatest,1,length(dof))./squeeze(omegastoreCOS(:,:,1,1).^.5);
        tLG=T^.5*repmat(betatest,1,length(dof))./squeeze(omegastoreLG(:,:,1,1).^.5);
        tSS=T^.5*repmat(betatest,1,length(dof))./squeeze(omegastoreSS(:,:,1,1).^.5);
        tKVB=T^.5*betatest./squeeze(omegastoreKVB(:,:,:,1,1).^.5);
        % Rejection rates
        QSpow(j,:)=mean((1-(QSmat1<tQS & tQS<QSmat2)));
        NWpow(j,:)=mean((1-(NWmat1<tNW & tNW<NWmat2)));
        EWPpow(j,:)=mean((1-(EWPmat1<tEWP & tEWP<EWPmat2)));
        COSpow(j,:)=mean((1-(COSmat1<tCOS & tCOS<COSmat2)));
        LGpow(j,:)=mean((1-(LGmat1<tLG & tLG<LGmat2)));
        SSpow(j,:)=mean((1-(SSmat1<tSS & tSS<SSmat2)));
        KVBpow(j,:)=mean((1-(KVBmat1<tKVB & tKVB<KVBmat2)));
    end
    powsample=[QSpow,NWpow,EWPpow,COSpow,LGpow,SSpow,KVBpow];%,EWPpow,EWPpow,EWPpow];
    
    %% Find worst case power
    grid=2:.05:4;
    samp=delrange;
    pow_gau = normcdf(-1.96+grid)+normcdf(-1.96-grid); % Oracle power
    powinterp=interp1(samp, powsample,grid,'linear'); % Interpolate power curves based on simulations
    [maxPloss]=max(repmat(pow_gau',1,length(MCsize))-powinterp); % Find maximum power loss
    
elseif m==2
    %% Size tests
    
    % Factors from...
    factor=(dof-m+1)./dof; % See Sun (2013)
    
    % Storage
    FQS=zeros(draws,length(dof));
    FNW=zeros(draws,length(dof));
    FEWP=zeros(draws,length(dof));
    FCOS=zeros(draws,length(dof));
    FLG=zeros(draws,length(dof));
    FSS=zeros(draws,length(dof));
    FKVB=zeros(draws,1);
    
    % Test statistics
    for i=1:draws
        for d=1:length(dof)
            FQS(i,d)=T*betadev(i,:)*squeeze(omegastoreQS(i,d,:,:))^-1*betadev(i,:)'/m;
            FNW(i,d)=T*betadev(i,:)*squeeze(omegastoreNW(i,d,:,:))^-1*betadev(i,:)'/m;
            FEWP(i,d)=factor(d).*T*betadev(i,:)*squeeze(omegastoreEWP(i,d,:,:))^-1*betadev(i,:)'/m;
            FCOS(i,d)=factor(d).*T*betadev(i,:)*squeeze(omegastoreCOS(i,d,:,:))^-1*betadev(i,:)'/m;
            FLG(i,d)=factor(d).*T*betadev(i,:)*squeeze(omegastoreLG(i,d,:,:))^-1*betadev(i,:)'/m;
            FSS(i,d)=factor(d).*T*betadev(i,:)*squeeze(omegastoreSS(i,d,:,:))^-1*betadev(i,:)'/m;
        end
        FKVB(i)=T*betadev(i,:)*squeeze(omegastoreKVB(i,1,:,:))^-1*betadev(i,:)'/m;
    end
    
    % Critical values
    QScrits=[5.6501    3.9263    3.3979    3.1943]; % Externally calculated, see readme
    QSmat1=repmat(QScrits(1,:),draws,1);
    NWcrits=[5.9790,4.2755,3.5859,3.2770]; % Externally calculated, see readme
    NWmat1=repmat(NWcrits(1,:),draws,1);
    KVBcrits=51.41/2; % From KVB paper (division by 2 from KVB note)
    KVBmat1=repmat(KVBcrits(1,:),draws,1);
    basisfncrits=finv(.95,2,dof-2+1); % Exact F for fixed-b
    basismat1=repmat(basisfncrits(1,:),draws,1);
    
    % Higher order critical values
    crita=repmat(finv(.95,m,dof-m+1),draws,1).*(1+pi^2/6*repmat((dof/T).^2,draws,1).*repmat(w2astore,1,length(dof)));
    critb=repmat(finv(.95,m,dof-m+1),draws,1).*(1+pi^2/6*repmat((dof/T).^2,draws,1).*repmat(w2bstore,1,length(dof)));
    critplug=repmat(finv(.95,m,dof-m+1),draws,1).*(1+pi^2/6*repmat((dof/T).^2,draws,1).*repmat(plugin,1,length(dof)));
    EWPcrita1=crita;
    EWPcritb1=critb;
    EWPcritplug1=critplug;
    
    % Rejection rates
    QSrej=1-mean(FQS<QSmat1);
    NWrej=1-mean(FNW<NWmat1);
    EWPrej=1-mean(FEWP<basismat1);
    COSrej=1-mean(FCOS<basismat1);
    LGrej=1-mean(FLG<basismat1);
    SSrej=1-mean(FSS<basismat1);
    KVBrej=1-mean(FKVB<KVBmat1);
    EWParej=1-mean(FEWP<EWPcrita1);
    EWPbrej=1-mean(FEWP<EWPcritb1);
    EWPplugrej=1-mean(FEWP<EWPcritplug1);
    MCsize=[QSrej,NWrej,EWPrej,COSrej,LGrej,SSrej,KVBrej];%,EWParej,EWPbrej,EWPplugrej];
    refsize=[EWParej,EWPbrej,EWPplugrej];
    
    % Quantiles
    QSquantiles=quantile(FQS,.95);
    QSmat1=repmat(QSquantiles(1,:),draws,1);
    NWquantiles=quantile(FNW,.95);
    NWmat1=repmat(NWquantiles(1,:),draws,1);
    EWPquantiles=quantile(FEWP,.95);
    EWPmat1=repmat(EWPquantiles(1,:),draws,1);
    COSquantiles=quantile(FCOS,.95);
    COSmat1=repmat(COSquantiles(1,:),draws,1);
    LGquantiles=quantile(FLG,.95);
    LGmat1=repmat(LGquantiles(1,:),draws,1);
    SSquantiles=quantile(FSS,.95);
    SSmat1=repmat(SSquantiles(1,:),draws,1);
    KVBquantiles=quantile(FKVB,.95);
    KVBmat1=repmat(KVBquantiles(1),draws,1);
    
    % Compute alternatives
    delrange=0:.25:4;
    if isequal(DGP,@stochasAR)
        rho=alpha^2;
        betas=delrange*sqrt((1+rho)/(1-rho))/sqrt(T);
    elseif isequal(DGP,@loc2)
        betas=delrange/(1-alpha)/sqrt(T);
    end
    
    % Construct Storage
    QSpow=NaN(length(delrange),4);
    NWpow=NaN(length(delrange),4);
    EWPpow=NaN(length(delrange),4);
    COSpow=NaN(length(delrange),4);
    LGpow=NaN(length(delrange),4);
    SSpow=NaN(length(delrange),4);
    KVBpow=NaN(length(delrange),1);
    FQS=zeros(draws,length(dof));
    FNW=zeros(draws,length(dof));
    FEWP=zeros(draws,length(dof));
    FCOS=zeros(draws,length(dof));
    FLG=zeros(draws,length(dof));
    FSS=zeros(draws,length(dof));
    FKVB=zeros(draws,1);
    
    % Test stats
    for j=1:length(delrange);
        betatest=betadev+[repmat(betas(j),draws,1),zeros(draws,1)]; % Augment estimated beta deviation from run with beta_alt to calculate power under alterntive
        for i=1:draws
            for d=1:length(dof)
                FQS(i,d)=T*betatest(i,:)*squeeze(omegastoreQS(i,d,:,:))^-1*betatest(i,:)'/m;
                FNW(i,d)=T*betatest(i,:)*squeeze(omegastoreNW(i,d,:,:))^-1*betatest(i,:)'/m;
                FEWP(i,d)=factor(d).*T*betatest(i,:)*squeeze(omegastoreEWP(i,d,:,:))^-1*betatest(i,:)'/m;
                FCOS(i,d)=factor(d).*T*betatest(i,:)*squeeze(omegastoreCOS(i,d,:,:))^-1*betatest(i,:)'/m;
                FLG(i,d)=factor(d).*T*betatest(i,:)*squeeze(omegastoreLG(i,d,:,:))^-1*betatest(i,:)'/m;
                FSS(i,d)=factor(d).*T*betatest(i,:)*squeeze(omegastoreSS(i,d,:,:))^-1*betatest(i,:)'/m;
                
            end
            FKVB(i)=T*betatest(i,:)*squeeze(omegastoreKVB(i,1,:,:))^-1*betatest(i,:)'/m;
        end
        
        % Rejection Rates
        QSpow(j,:)=1-mean(FQS<QSmat1);
        NWpow(j,:)=1-mean(FNW<NWmat1);
        EWPpow(j,:)=1-mean(FEWP<EWPmat1);
        COSpow(j,:)=1-mean(FCOS<COSmat1);
        LGpow(j,:)=1-mean(FLG<LGmat1);
        SSpow(j,:)=1-mean(FSS<SSmat1);
        KVBpow(j,:)=1-mean(FKVB<KVBmat1);
    end
    powsample=[QSpow,NWpow,EWPpow,COSpow,LGpow,SSpow,KVBpow];%,EWPpow,EWPpow,EWPpow];
    
    %% Find worst case power
    grid=0:.05:4;
    samp=delrange;
    pow_gau=1-ncx2cdf(5.9915,2,grid.^2); % Oracle power
    powinterp=interp1(samp, powsample,grid,'linear'); % Interpolate curves
    [maxPloss]=max(repmat(pow_gau',1,length(MCsize))-powinterp); % Find maximum loss
    
end


end
